﻿using System;
using System.Runtime.CompilerServices;

namespace _01OperatorOverloading
{
    class Bruch
    {
        private int z, n;
        public Bruch(int Z, int N = 1)
        {
            z = Z;
            if (N == 0)
                throw new ArgumentNullException("Nenner darf nicht gleich 0 sein!");
            n = N;
        }
        public Bruch(string t)
        {
            string[] d = t.Split('/');
            z = Convert.ToInt32(d[0]);
            n = Convert.ToInt32(d[1]);
        }

        public static Bruch Mult1(Bruch b1, Bruch b2)
        {
            int zaehler = b1.z * b2.z;
            int nenner = b1.n * b2.n;

            //b1.z = zaehler;
            //b1.n = nenner;
            Bruch erg = new Bruch(zaehler, nenner);
            return erg;
        }
        public static Bruch Mult2(Bruch b1, Bruch b2)
        {
            return new Bruch(b1.z * b2.z, b1.n * b2.n);
        }
        public static Bruch Mult(Bruch b1, Bruch b2)
            => new Bruch(b1.z * b2.z, b1.n * b2.n);
        public static Bruch operator *(Bruch b1, Bruch b2)
            => new Bruch(b1.z * b2.z, b1.n * b2.n);
        public static bool operator ==(Bruch b1, Bruch b2)
            => b1.z == b2.z && b1.n == b2.n;
        // nur wenn b1.z == b2.z ist, wird b1.n == b2.n ausgeführt
        public static bool operator !=(Bruch b1, Bruch b2)
            => !(b1 == b2);
        public static Bruch operator <<(Bruch b, int x) => new Bruch(b.z * x, b.n * x);
        public static Bruch operator >>(Bruch b, int x) => new Bruch(b.z / x, b.n / x);


        // Konvertierungsoperatoren:
        public static explicit operator int(Bruch b)
            => b.z / b.n;
        public static explicit operator double(Bruch b)
            => (double)b.z / (double)b.n;
        public static implicit operator Bruch(int zahl)
            => new Bruch(zahl);

        // Überladung unärer Operatoren:
        public static Bruch operator +(Bruch b) => b;
        public static Bruch operator -(Bruch b) => new Bruch(-b.z, b.n);

        // ACHTUNG: Nur EINE Inkrement-/Dekrement- Implementierung angeben!
        // Post- oder Pre-Inkrement/Dekrement werden vom Compiler generiert!
        // Die folgenden beiden Zeilen sind also falsch!
        //public static Bruch operator ++(Bruch b) { b.z += b.n; return b; }
        //public static Bruch operator --(Bruch b) { b.z -= b.n; return b; }

        // !!! So geht es richtig:    x = 5; cw(x++)--> cw(x); x=x+1  ,  cw(++x) --> x=x+1;cw(x)
        public static Bruch operator ++(Bruch b) { return new Bruch(b.z + b.n, b.n); }
        public static Bruch operator --(Bruch b) { return new Bruch(b.z - b.n, b.n); }
        public override string ToString() => $"{z} / {n}";


    }


    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine(2 << 3);  // 10 << 3 --> 10 000
            Console.WriteLine("***************");
            Bruch b1 = new Bruch(2, 3);
            Bruch b2 = new Bruch(2);
            Bruch b3 = new Bruch("3/4");

            // Bruch e1 = b1.Mult(b2);   // wäre möglich
            // Bruch e1 = b1.Mult(2);    // ist möglich
            // Bruch e1 = 3.Mult(b1);    // ist nicht möglich, da 3 vom Typ int und nicht Bruch ist
            Bruch e2 = Bruch.Mult(b1, b2);
            //Bruch e2 = Bruch.Mult(2,  b2);  // nicht nötig, über Konv.Op. erledigt
            //Bruch e2 = Bruch.Mult(b1, 2);   // nicht nötig, über Konv.Op. erledigt

            Bruch e3 = b1 * b2;
            Bruch e4 = b1 * 2;
            Bruch e5 = 2 * b1;

            // Wir haben Bruch.operator*(Bruch, Bruch)
            // int.op*, double.op*, Bruch.op*
            // Bruch.op*(Bruch, Bruch)
            //      int->Bruch, Bruch   ???
            // 2 --> impl.Konv. --> Bruch
            // op* (Bruch, Bruch)
            // double.op*(int, double)  --> double.op*(int-->double, double)
            int k2 = 3 * (int)4.5;

            int k = (int)e3;   // explizite Konvertierung
            double d = (double) e3;      
            Console.WriteLine(k);
            Console.WriteLine(d);

            double d2 = (int)34;
            int k3 = (int)3.5;

            Bruch b4 = new Bruch(2, 5);         // b4: neuer Speicher #100.000
            Bruch b5 = new Bruch(2, 5);         // b5: neuer Speicher #200.000
            if (b4 == b5)
                Console.WriteLine("Sind gleich");
            else
                Console.WriteLine("Sind unterschiedlich");

            Bruch b6 = new Bruch(1, 2);
            Console.WriteLine(b6++);
            Console.WriteLine(++b6);

            int x = 55;
            Console.WriteLine(x++);
            Console.WriteLine(++x);

            Bruch b7 = new Bruch(1, 3);
            Console.WriteLine(b7 << 2);

            Bruch b8 = new Bruch(2, 4);
            Console.WriteLine(-b8);
        }
    }
}
