﻿using System;
using System.Collections;
using System.Collections.Generic;


namespace _02VerketteteListe_2
{
    class StringListe : IEnumerable
    {
        // Die Klasse Element ist nun eine *private* Klasse der Klasse StringListe
        // Die Methoden von StringListe dürfen auf private Member zugreifen
        // Methoden außerhalb von StringListe dürfen dies nicht!
        private class Element  // Element ist eine private Hilfsklasse der Klasse StringListe
        {
            public string name;
            public Element next = null;
            public Element(string name) { this.name = name; }
            public override string ToString() => name;
        }

        Element anf = null, ende = null;
        int anz = 0;      // Sehr sinnvoll für Index-Ops
        //public StringListe() { }
        public void AddEnd(string name)
        {
            Element neuesElement = new Element(name);   // 1. Neues Element anlegen
            anz++;
            if (anf == null)                            // 2. Leere Liste?
                anf = ende = neuesElement;
            else
            {
                ende.next = neuesElement;               // 3. Neues Element am Ende anfügen
                ende = ende.next;
            }
        }
        public void AddFront(string name)
        {
            // 1. Leere Liste
            // 2. Es existiert mind. ein Element
            Element neuesElement = new Element(name);   // Neues Element anlegen
            anz++;
            if (anf == null)                            // 1.Fall: Leere Liste?
                anf = ende = neuesElement;
            else
            {
                neuesElement.next = anf;                // 2. Fall
                anf = neuesElement;
            }
        }
        public void AddBeforeNth(int index, string name)
        {
            if (index < 0 || index > anz)
                throw new ArgumentOutOfRangeException("Index außerhalb der Listengröße");
            if (index == 0)
                AddFront(name);
            else if (index == anz)
                AddEnd(name);
            else
            {
                Element neu = new Element(name);
                anz++;
                Element item = anf;
                for (int i = 1; i < index; i++)
                    item = item.next;
                neu.next = item.next;    // Die Reihenfolge dieses und des nächsten Befehls ist ENTSCHEIDEND!
                item.next = neu;
            }
        }
        public void AddSorted(string name)
        {
            // 1. Fall: Leere Liste oder Anfügen am Listenende
            if (anf == null || ende.name.CompareTo(name) <= 0)
                AddEnd(name);
            else
            {
                if (name.CompareTo(anf.name) <= 0)
                    AddFront(name);
                else
                {
                    // Wir wissen: Das neue Element ist nicht das erste und nicht das letzte Element
                    Element neu = new Element(name);
                    anz++;

                    Element item = anf;
                    while (item.next.name.CompareTo(name) < 0)
                        item = item.next;
                    neu.next = item.next;
                    item.next = neu;
                }
            }
        }
        public bool Suche(string name)
        {
            for (Element item = anf; item != null; item = item.next)
            {
                if (item.name == name)
                    return true;
            }
            return false;
        }
        public void Print()
        {
            for (Element item = anf; item != null; item = item.next)
            {
                Console.WriteLine(item);
            }
        }
        public IEnumerator GetEnumerator()
        {  
            for (Element item = anf; item != null; item = item.next)
            {
                yield return item.name; // Merken dieser Ausführungsposition
                                        // UND Zurückliefern von item.name
                                        // Beim nächsten Aufruf von GetEnumerator() wird
                                        // an der gespeicherten Pos. weitergemacht.
            }
        }
        public IEnumerable<string> MeinFilter(string pattern)
        {
            for (Element item = anf; item != null; item = item.next)
            {
                if (item.name.Contains(pattern))
                    yield return item.name; // Merken dieser Ausführungsposition
                                            // UND Zurückliefern von item.name
                                            // Beim nächsten Aufruf von GetEnumerator() wird
                                            // an der gespeicherten Pos. weitergemacht.
            }
        }
        public IEnumerable Iterate(int offset = 0)
        {
            Element item = anf;
            for (int i = 0; i < offset; i++)
                item = item.next;
            while (item != null)
            {
                yield return item.name;
                item = item.next;
            }
        }

        // foreach --> GetEnumerator() merkt sich die Pos nach yield return und liefert ("Anton")
        // item = "Anton"
        // foreach --> GetEnumerator(), überprüft, ob noch eine Ausführungspos. gespeichert ist
        //                  JA: --> hier geht die Ausführung weiter
        //                          item=item.next, merkt sich die Pos nach yield return und liefert ("Berta")
        // item = "Berta"
        // ... bis GetEnumerator() am Methodenende angelangt ist.
        public void DeleteFirst()
        {
            // 1.Fall: Liste ist leer
            // 2.Fall: Liste besteht nur aus einem Element
            // 3.Fall: Liste hat mehr als ein Element
            if (anf == null)  // 1. Fall
                return;
            anz--;
            if (anf == ende)  // 2. Fall
                anf = ende = null;
            else
                anf = anf.next;   // 3. Fall
        }
        public void DeleteLast()
        {
            // 1.Fall: Liste ist leer
            // 2.Fall: Liste besteht nur aus einem Element
            // 3.Fall: Liste hat mehr als ein Element
            if (anf == null)  // 1. Fall
                throw new NullReferenceException("Die Liste ist leer");
            anz--;
            if (anf == ende)  // 2. Fall
                anf = ende = null;
            else              // 3. Fall
            {  // Wir wissen: Die Liste hat mehr als ein Element, d.h. es gibt ein
               // vorletztes Element vorletzter.next == ende
                Element vorletzter = anf;
                while (vorletzter.next != ende)
                    vorletzter = vorletzter.next;
                vorletzter.next = null;
                ende = vorletzter;
            }
        }
        public void DeleteNth(int index)
        {
            // 1.Fall: Liste ist leer ODER Ind>=anz --> Fehler
            // 2.Fall: Ind == 0  --> DeleteFirst
            // 3.Fall: Ind == anz-1 --> DeleteLast
            // 4. Bis hierher: Liste hat mind. ein Element und das zu löschende Element
            //                 ist nicht das Erste oder Letzte
            if (anz == 0 || index >= anz)  // 1. Fall
                throw new ArgumentOutOfRangeException("Außerhalb der Anzahl der Listenelemente");
            if (index == 0)  // 2. Fall
                DeleteFirst();
            else if (index == anz - 1) // 3. Fall
                DeleteLast();
            else
            {
                Element vorletzter = anf;
                for (int i = 0; i < index - 1; i++)  // Liste bis zum n-1-ten Element durchlaufen
                    vorletzter = vorletzter.next;
                vorletzter.next = vorletzter.next.next;
                anz--;
            }
        }
        public void DeleteByName(string name)
        {
            // leere Liste ohne Elemente
            if (anf == null)
                return;
            // erstes Element = gesuchtes Element
            if (anf.name.CompareTo(name) == 0)
                DeleteFirst();
            // Wir wissen jetzt:
            // Liste hat mind. ein Element UND das erste Element ist nicht das gesuchte Element
            else
            {
                Element item = anf;
                // Durchlaufen der Liste
                // solange Elemente in Liste vorhanden und gesuchtes Element noch nicht gefunden
                while (item.next != null && item.next.name.CompareTo(name) != 0)
                    item = item.next;
                // Wenn gesuchtes Element in Liste vorhanden
                // item = "B"
                if (item.next != null)  // "C"
                {
                    anz--;
                    // Nächstes Element überspringen = löschen
                    item.next = item.next.next;  // item.next = null
                    // Wenn gesuchtes Element == letztes Element:
                    if (item.next == null)
                        ende = item;
                }
            }
        }
        private Element NthElem(int index)
        {
            if (index < 0 || index >= anz)
                throw new IndexOutOfRangeException("Listenindex außerhalb des gültigen Bereichs");
            Element item = anf;
            while (index > 0)
            {
                item = item.next;
                index--;
            }
            return item;
        }
        public string this[int index]
        {
            get => NthElem(index).name;
            set { NthElem(index).name = value; }
        }
        public StringListe Reverse() // DIES IST NICHT DIE LÖSUNG FÜR DIE PRAKTIKUMSAUFGABE!!!
        {
            StringListe neueListe = new StringListe();
            for (Element item = anf; item != null; item = item.next)
                neueListe.AddFront(item.name);
            return neueListe;
        }
    }
    class Program
    {
        static void Main(string[] args)
        {
            StringListe l1 = new StringListe();
            
            l1.AddSorted("Emil ");
            l1.AddSorted("Anton");
            l1.AddSorted("Berta");
            l1.AddSorted("Claudia");
            l1.AddSorted("Dieter");
            #region weg
            //l1.Print();
            //l1 = l1.Reverse();
            //Console.WriteLine("==================");
            //l1.Print();
            //Console.WriteLine("==================");

            //Console.WriteLine(l1[3]);
            //Console.WriteLine("==================");
            ////l1.DeleteFirst();
            ////l1.DeleteLast();
            ////l1.DeleteNth(1);
            ////l1.Print();
            //l1.AddBeforeNth(2, "Bodo");
            //l1.Print();
            #endregion
            foreach (var item in l1.MeinFilter("er"))
            {
                Console.WriteLine($"foreach: {item}");
            }

            //var res = from w in l1
            //          where w.Length < 6
            //          orderby w descending
            //          select new { OrgName = w, GrossName = w.ToUpper(), Länge = w.Length };

            //foreach (var r in res)
            //{
            //    Console.WriteLine(r);
            //}
        }
    }
}
