﻿using System;

namespace _05_InterfaceIntro
{
    interface IBesteuerbareObjekte
    {
        /* public abstract */double Steuern();
    }
    interface IVerkaufbareObjekte
    {
        void Verkaufen();
    }

    abstract class Kfz : IBesteuerbareObjekte, IVerkaufbareObjekte
        // abstract:
        // 1) Von dieser Klasse können keine Objekte angelegt werden (manchmal eine wünschenswerte Eigenschaft)
        // 2) Die Klasse enthält eine oder mehrere abstrakte Methode(n) und muss deshalb als abstract def. werden
    {
        public abstract double Steuern();   // Steuern() und Verkaufen() müssen wieder als abstrakt definiert sein,
        public abstract void Verkaufen();   // da ansonsten die Interface-Klassen nicht implementiert werden
        public void TueEtwas() { }
    }
    class Pkw : Kfz
    {
        double hubraum;

        // Steuern() und Verkaufen() müssen in dieser nicht-abstrakten Klasse implementiert
        // werden, da ansonsten nicht alle abstrakten Methoden implementiert sind
        public override double Steuern() => (hubraum + 99) / 100 * 9.50;
        public override void Verkaufen()
        {
            Console.WriteLine("PKW wird verkauft");
            TueEtwas();
        }
    }
    class Pflanzen : IVerkaufbareObjekte
    {
        public void Verkaufen()
        {
            Console.WriteLine("Pflanze wird verkauft");
        }
    }
    class Immobilien : IBesteuerbareObjekte, IVerkaufbareObjekte
    {
        // Steuern() und Verkaufen() müssen direkt implementiert werden, da ansonsten
        // nicht alle abstrakten Methoden implementiert sind
        public double Steuern() => 1000;
        public void Verkaufen() { Console.WriteLine("Haus wird verkauft"); }
    }

    class Program
    {
        static void Main(string[] args)
        {
            IBesteuerbareObjekte[] bObj = new IBesteuerbareObjekte[3];
            Immobilien i1 = new Immobilien();
            Pkw p1 = new Pkw();
            bObj[0] = i1;
            bObj[1] = p1;
            //bObj[2] = new Pflanzen();

            IVerkaufbareObjekte[] vObj = new IVerkaufbareObjekte[2];
            vObj[0] = i1;
            vObj[1] = p1;
            vObj[2] = new Pflanzen();




            //i1.Verkaufen();
            i1.Steuern();

            p1.Verkaufen();
            p1.Steuern();

            if (p1 is Kfz) Console.WriteLine("p1 ist Kfz");
            if (p1 is IBesteuerbareObjekte) Console.WriteLine("p1 ist besteuerbar");
            if (p1 is IVerkaufbareObjekte) Console.WriteLine("p1 ist verkaufbar");

            IBesteuerbareObjekte o1 = (IBesteuerbareObjekte)p1;
            // Pkw ist Kfz ist IBesteuerbar
            // aber nicht umgekehrt für alle Objekte, z.B.
            // Immobilie ist IBesteuerbar, aber kein Kfz
            //Pkw p2 = (Pkw)i1;

        }
    }
}
