﻿using System;

/*
 * Gegeben ist das folgende Programm als Grundgerüst eines Nachrichtendienstes. 
 * Dabei soll gelten: Ein Nachrichtenserver bietet eine Schnittstelle (Delegate), an der beliebig viele 
 * Nachrichtenempfangsmethoden anderer Klassen andocken können. Bevor Nachrichten jedoch versendet werden, 
 * können diese mit Hilfe einer oder mehrerer kundenspezifischer Modifikationsfunktionen bearbeitet werden 
 * (hier wir als Beispiel ein alter Firmenname durch den neuen Firmennamen ersetzt sowie an die Nachrichten 
 * wird noch ein Anhängsel (Signatur) angehängt. 
 */
namespace _10_UebgMsgServer
{
    delegate void Callback(string notifier);
    delegate void Transform(ref string msg);
    class MsgServer
    {
        public event Callback Notify;
        public event Transform Transformations;

        public void SendMsg(string msg)
        {
            Console.WriteLine($"\n\n\nOriginal-Nachricht: {msg}");
            Transformations?.Invoke(ref msg);
            // Alle in Transformations gespeicherten Transformationen anwenden
            Notify?.Invoke(msg);
            // Den "transformierten" String an alle in Notify gespeicherten Listener 
            // versenden
        }
    }
    class Client
    {
        public string Name { get; }
        public Client(string Name) { this.Name = Name; }
        public void Listener(string text)
        {
            Console.WriteLine($"  {Name}: {text}");
        }
        // ToDo: Statische "Listener" Methode
    }
    class Program
    {
        static void Signature(ref string text)
        {
            text += "\nMessage via Magic MsgServer, AAA Corporation";
        }
        static void OldNewCorp(ref string text)  // ohne ref: Es wird eine Kopie der string-Referenz übergeben
                                                 // mit ref: Es wird die Referenz auf die Variable aus der
                                                 // aufrufenden Methode übergeben
        {
            text = text.Replace("ZZZ Corporation", "AAA Corporation");
        }
        static void NewMethod(ref string text)
        {
            text = text.Replace("Kunden", "MITARBEITER");
        }
        // ToDo: Weitere Transformationsregel

        static void Main(string[] args)
        {
            MsgServer server = new MsgServer();
            server.Transformations += OldNewCorp;    // msg --> OldNewCorp  --> Signature
            server.Transformations += Signature;
            server.Transformations += NewMethod;

            Client mueller = new Client("Müller");
            Client meier = new Client("Meier");
            Client huber = new Client("Huber");

            server.Notify += mueller.Listener;
            server.Notify += meier.Listener;
            //server.Notify += Console.WriteLine;

            server.SendMsg("Herzlich Willkommen, liebe Kunden! ZZZ Corporation");

            server.Notify += huber.Listener;
            server.Notify -= mueller.Listener;
            server.Transformations -= Signature;
            server.SendMsg("Wir wünschen einen erfolgreichen Tag, liebe Kunden, ZZZ Corporation");
        }
    }
}
