﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;

namespace _11BinTreePersVerwaltung2
{
    class ListStack<TYP> 
    {
        class LElem<TElem>
        {
            public TElem d;
            public LElem<TElem> nxt;

            public LElem(TElem Data, LElem<TElem> Next) { d = Data; nxt = Next; }
        }

        LElem<TYP> root = null;
        int count = 0;
        public int Count
        {
            get { return count; }
        }
        public void Push(TYP Data)
        {
            LElem<TYP> le = new LElem<TYP>(Data, root);
            root = le;
            count++;
        }
        /// <summary>
        /// Liefert das nächste Element des Stacks zurück
        /// </summary>
        /// <returns></returns>
        public TYP Pop()
        {
            if (count == 0)
                throw new IndexOutOfRangeException("Stack empty");
            count--;
            LElem<TYP> le = root;
            root = root.nxt;
            le.nxt = null;
            return le.d;
        }
    }

    /// <summary>
    /// Generischer binärer Baum
    /// </summary>
    /// <typeparam name="T"></typeparam>
    class BinTree<T> : IEnumerable
        where T : IComparable<T>
    {
        /// <summary>
        /// Datenstruktur für einen Knoten des binären Baums
        /// </summary>
        /// <typeparam name="NT"></typeparam>
        class Node
        {
            public T data;
            public Node left = null, right = null;

            public Node(T Data) { data = Data; }
            /// <summary>
            /// Hilfsfunktion zum Traversieren des binären Baums. Liefert die Daten für die
            /// foreach-Schleife zurück
            /// </summary>
            /// <returns></returns>
            /// 
            public IEnumerable<T> enumerate()
            {
                if (left != null)
                    foreach (T item in left.enumerate())
                        yield return item;
                yield return data;
                if (right != null)
                    foreach (T item in right.enumerate())
                        yield return item;
            }
            /// <summary>
            /// Gibt die Daten des binären Baums rekursiv aus
            /// </summary>
            public void print()   // Methode der Klasse Node
            {
                if (left != null)
                    left.print();
                Console.WriteLine(this.data);
                if (right != null)
                    right.print();
            }
        }
        Node root = null;
        public void Print()
        {
            if (root != null)
                root.print();
        }

        public IEnumerator GetEnumerator()
        {
            foreach (T item in root.enumerate())
                yield return item;
        }

        public void Insert(T Data)
        {
            Node neu = new Node(Data);
            if (root == null)           // Gibt es schon einen Baum?
                root = neu;               // Nein: Neues Element ist Wurzel des Baums
            else
            {
                Node tmp = root;        
                bool found = false;
                do      // Solange die Einfügeposition noch nicht gefunden ist, tue ...
                {
                    if (neu.data.CompareTo(tmp.data) < 0)
                    {   // links oder rechts von dem aktuellen Element einfügen?
                        // LINKS
                        if (tmp.left == null)   
                        {                   // Ist der linke Ast noch leer?
                            tmp.left = neu;   // Dann hier einfügen
                            found = true;
                        }
                        else
                            tmp = tmp.left; // Sonst dem linken Ast folgen
                    }
                    else
                    {       // RECHTS
                        if (tmp.right == null)
                        {                   // Ist der rechte Ast noch leer?
                            tmp.right = neu;  // Dann hier einfügen
                            found = true;
                        }
                        else                // Sonst dem rechten Ast folgen
                            tmp = tmp.right;
                    }
                } while (!found);
            }
        }
    }
    
}
