﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace _11Enumerable
{
    /*
     * foreach benötigt IEnumerator GetEnumerator --> Rückgabetyp object oder 
     *                  IEnumerator<T> GetEnumerator()  --> Rückgabetyp T
     * IEnumerable-Interface enthält nur eine Methode: IEnumerator GetEnumerator();
     * IEnumerable<T> enthält die Methode IEnumerator<T> GetEnumerator(), 
     *                erbt aber auch IEnumerable, benötigt somit ZUSÄTZLICH IEnumerator GetEnumerator
     * LINQ benötigt IEnumerable, akzeptiert aber auch IEnumerable<T>, da dies ja auch
     *                IEnumerator GetEnumerator enthält
     * Bei Angabe von :IEnumerable<T> verwenden sowohl foreach als auch LINQ IEnumerator<T> GetEnumerator
     * -->  IEnumerator oder IEnumerator<T> bei GetEnumerator
     * -->  IEnumerable oder IEnumerable<T> bei eigenen Aufzählungsfunktionen
     */
    class Aufzaehlung<T> : IEnumerable<T>
    {
        T[] data;
        public Aufzaehlung(params T[] data) { this.data = data; }

        public IEnumerator<T> GetEnumerator()
        {
            Console.WriteLine("Generic GetEnumerator");
            foreach (var item in data)
                yield return item;
        }
        IEnumerator IEnumerable.GetEnumerator()
        {
            Console.WriteLine("IEnumerable.GetEnumerator");
            foreach (var item in data)
                yield return item;
        }

        public IEnumerable<T> meineAufzaehlung()
        {
            foreach (var item in data)
                yield return item;
        }
    }
    class Program
    {
        static void Main(string[] args)
        {
            Aufzaehlung<string> aufzaehlung = new Aufzaehlung<string>( "Dieter", "Berta", "Anton", "Erika", "Carl");
            foreach (var item in aufzaehlung)
                Console.WriteLine(item);

            var res = from string n in aufzaehlung
                      where  n.Contains("er")
                      orderby n
                      select new { Name = n };
            Console.WriteLine("Namen: ");
            foreach (var p in res)
                Console.WriteLine(p);
        }
    }
}
